module CharRef(CharRef(..), CharMap, (!), lookup, update, insert, fromList, map, elems) where

import Prelude hiding(lookup, map)
import qualified Prelude
import qualified Data.IntMap as IM

data CharRef = CR {-# UNPACK #-} !Int !(Maybe String)

instance Show CharRef where
  show (CR n Nothing) = "[" ++ show n ++ "]"
  show (CR _ (Just name)) = name

instance Eq CharRef where
  CR x _ ==  CR y _ = x == y

instance Ord CharRef where
  compare (CR x _) (CR y _) = compare x y

newtype CharMap a = CM (IM.IntMap a)
  deriving (Show, Eq)

{-# INLINE (!) #-}
{-# INLINE lookup #-}
{-# INLINE insert #-}

(!) :: CharMap a -> CharRef -> a
CM m ! CR x _ = m IM.! x

lookup :: CharRef -> CharMap a -> Maybe a
lookup (CR x _) (CM m) = IM.lookup x m

update :: (a -> Maybe a) -> CharRef -> CharMap a -> CharMap a
update f (CR x _) (CM m) = CM $ IM.update f x m

insert :: CharRef -> a -> CharMap a -> CharMap a
insert (CR x _) val (CM m) = CM $ IM.insert x val m

fromList :: [(CharRef, a)] -> CharMap a
fromList list = CM $ IM.fromList $ Prelude.map tr list
  where
    tr (CR x _, val) = (x, val)

map :: (a -> b) -> CharMap a -> CharMap b
map f (CM m) = CM $ IM.map f m

elems :: CharMap a -> [a]
elems (CM m) = IM.elems m
