# ifndef M100SIM_MATCH_H
# define M100SIM_MATCH_H

# include <stdio.h>

# ifdef __cplusplus
extern "C"
{
# endif

/* abstract sword reference */
typedef struct m100sim_sword_info *m100sim_sword_ref;

typedef struct
{
  const char *name; /* may be null */
  int hp;
  int atk;
  int def;
  int spd;
  int n_swords;
  const m100sim_sword_ref *swords;
} m100sim_character;

typedef struct
{
  FILE *log_output; /* may be null */
  const char *line_terminator;
} m100sim_match_options;

typedef enum
{
  m100sim_left_wins = 0,
  m100sim_right_wins = 1,
  m100sim_draw = 2,
  m100sim_aborted = 3
} m100sim_match_result;

m100sim_match_result m100sim_match(m100sim_match_options opts, m100sim_character left, m100sim_character right);

/* 
 * constructors of m100sim_sword_ref
 */

/* basic */
extern const m100sim_sword_ref m100sim_s_slash;
extern const m100sim_sword_ref m100sim_s_rapid;
extern const m100sim_sword_ref m100sim_s_heavy;
extern const m100sim_sword_ref m100sim_s_shock;
extern const m100sim_sword_ref m100sim_s_poison;
extern const m100sim_sword_ref m100sim_s_absolute;
extern const m100sim_sword_ref m100sim_s_magical;
extern const m100sim_sword_ref m100sim_s_death;
extern const m100sim_sword_ref m100sim_s_heat;
extern const m100sim_sword_ref m100sim_s_protect;
extern const m100sim_sword_ref m100sim_s_accel;
extern const m100sim_sword_ref m100sim_s_shield;
extern const m100sim_sword_ref m100sim_s_mirror;
extern const m100sim_sword_ref m100sim_s_life;
m100sim_sword_ref m100sim_s_recovery(int n);

/* m100 */
extern const m100sim_sword_ref m100sim_s_null;
extern const m100sim_sword_ref m100sim_s_spark;
extern const m100sim_sword_ref m100sim_s_firecracker;
extern const m100sim_sword_ref m100sim_s_eruption;
extern const m100sim_sword_ref m100sim_s_devour;
extern const m100sim_sword_ref m100sim_s_disaster;
extern const m100sim_sword_ref m100sim_s_perish;
extern const m100sim_sword_ref m100sim_s_finale;
extern const m100sim_sword_ref m100sim_s_combo;
extern const m100sim_sword_ref m100sim_s_wave;
extern const m100sim_sword_ref m100sim_s_dance;
extern const m100sim_sword_ref m100sim_s_scatter;
extern const m100sim_sword_ref m100sim_s_toxic;
extern const m100sim_sword_ref m100sim_s_abortion;
extern const m100sim_sword_ref m100sim_s_destroy;
extern const m100sim_sword_ref m100sim_s_battle;
extern const m100sim_sword_ref m100sim_s_confusion;
extern const m100sim_sword_ref m100sim_s_bloodsuck;
extern const m100sim_sword_ref m100sim_s_robbery;
extern const m100sim_sword_ref m100sim_s_dark;
extern const m100sim_sword_ref m100sim_s_antimagic;
extern const m100sim_sword_ref m100sim_s_overweight;
extern const m100sim_sword_ref m100sim_s_solid;
extern const m100sim_sword_ref m100sim_s_sonic;
extern const m100sim_sword_ref m100sim_s_light;
m100sim_sword_ref m100sim_s_rage(int n);
extern const m100sim_sword_ref m100sim_s_cool;
extern const m100sim_sword_ref m100sim_s_decay;
extern const m100sim_sword_ref m100sim_s_decel;
extern const m100sim_sword_ref m100sim_s_gentle;
extern const m100sim_sword_ref m100sim_s_flashfire;
extern const m100sim_sword_ref m100sim_s_alert;
extern const m100sim_sword_ref m100sim_s_evasion;
extern const m100sim_sword_ref m100sim_s_escutcheon;
extern const m100sim_sword_ref m100sim_s_amnion;
extern const m100sim_sword_ref m100sim_s_barrier;
extern const m100sim_sword_ref m100sim_s_soul;
extern const m100sim_sword_ref m100sim_s_push;
extern const m100sim_sword_ref m100sim_s_drive;
extern const m100sim_sword_ref m100sim_s_wriggle;
extern const m100sim_sword_ref m100sim_s_attraction;
extern const m100sim_sword_ref m100sim_s_first;
extern const m100sim_sword_ref m100sim_s_second;
extern const m100sim_sword_ref m100sim_s_communication;
m100sim_sword_ref m100sim_s_heal(int n);
extern const m100sim_sword_ref m100sim_s_chant;
extern const m100sim_sword_ref m100sim_s_prayer;
m100sim_sword_ref m100sim_s_ritual(int n);
extern const m100sim_sword_ref m100sim_s_prospect;
extern const m100sim_sword_ref m100sim_s_dragonize;
extern const m100sim_sword_ref m100sim_s_limit;
extern const m100sim_sword_ref m100sim_s_suppression;
m100sim_sword_ref m100sim_s_delay(int n);
m100sim_sword_ref m100sim_s_sheathe(int n);
m100sim_sword_ref m100sim_s_derive(int n);
extern const m100sim_sword_ref m100sim_s_vibration;
extern const m100sim_sword_ref m100sim_s_toxicant;
extern const m100sim_sword_ref m100sim_s_diffuse;
extern const m100sim_sword_ref m100sim_s_train;
m100sim_sword_ref m100sim_s_recurrence(int n);
m100sim_sword_ref m100sim_s_resonance(int n);
extern const m100sim_sword_ref m100sim_s_fictional;
extern const m100sim_sword_ref m100sim_s_multi;
extern const m100sim_sword_ref m100sim_s_replica;
extern const m100sim_sword_ref m100sim_s_reverb;
extern const m100sim_sword_ref m100sim_s_echo;
extern const m100sim_sword_ref m100sim_s_bubble;
extern const m100sim_sword_ref m100sim_s_spirit;
extern const m100sim_sword_ref m100sim_s_fetal;
extern const m100sim_sword_ref m100sim_s_sawhair;
extern const m100sim_sword_ref m100sim_s_sawa;
extern const m100sim_sword_ref m100sim_s_purple;
extern const m100sim_sword_ref m100sim_s_holand;
extern const m100sim_sword_ref m100sim_s_servant;
extern const m100sim_sword_ref m100sim_s_leg;
extern const m100sim_sword_ref m100sim_s_dog;
extern const m100sim_sword_ref m100sim_s_ko;
extern const m100sim_sword_ref m100sim_s_yoshi;
extern const m100sim_sword_ref m100sim_s_holy;
extern const m100sim_sword_ref m100sim_s_geki;
extern const m100sim_sword_ref m100sim_s_emperor;
extern const m100sim_sword_ref m100sim_s_bone;
extern const m100sim_sword_ref m100sim_s_princess;
extern const m100sim_sword_ref m100sim_s_summon;
extern const m100sim_sword_ref m100sim_s_age;

/* m2011a */
extern const m100sim_sword_ref m100sim_s_recoveryA;
extern const m100sim_sword_ref m100sim_s_perishA;
extern const m100sim_sword_ref m100sim_s_rageA;
extern const m100sim_sword_ref m100sim_s_fang;
extern const m100sim_sword_ref m100sim_s_two;
extern const m100sim_sword_ref m100sim_s_three;
extern const m100sim_sword_ref m100sim_s_four;
extern const m100sim_sword_ref m100sim_s_five;
extern const m100sim_sword_ref m100sim_s_six;
extern const m100sim_sword_ref m100sim_s_seven;
extern const m100sim_sword_ref m100sim_s_abortionA;
extern const m100sim_sword_ref m100sim_s_coolA;
extern const m100sim_sword_ref m100sim_s_decayA;
extern const m100sim_sword_ref m100sim_s_decelA;
extern const m100sim_sword_ref m100sim_s_darkA;
extern const m100sim_sword_ref m100sim_s_antimagicA;
extern const m100sim_sword_ref m100sim_s_pushA;
extern const m100sim_sword_ref m100sim_s_attractionA;
extern const m100sim_sword_ref m100sim_s_healA;
extern const m100sim_sword_ref m100sim_s_chantA;
extern const m100sim_sword_ref m100sim_s_rogation;
extern const m100sim_sword_ref m100sim_s_ritualA;
extern const m100sim_sword_ref m100sim_s_blaze;
extern const m100sim_sword_ref m100sim_s_deriveA;
extern const m100sim_sword_ref m100sim_s_slow;
extern const m100sim_sword_ref m100sim_s_danger;
extern const m100sim_sword_ref m100sim_s_recurrenceA;
extern const m100sim_sword_ref m100sim_s_resonanceA;
extern const m100sim_sword_ref m100sim_s_ageA;
extern const m100sim_sword_ref m100sim_s_mkbubble;
extern const m100sim_sword_ref m100sim_s_mkfence;
extern const m100sim_sword_ref m100sim_s_mkgate;
extern const m100sim_sword_ref m100sim_s_mkwall;
extern const m100sim_sword_ref m100sim_s_mkice;
extern const m100sim_sword_ref m100sim_s_mkmist;
extern const m100sim_sword_ref m100sim_s_mktear;

/* m2011 */
extern const m100sim_sword_ref m100sim_s_limb;
extern const m100sim_sword_ref m100sim_s_artisan;
extern const m100sim_sword_ref m100sim_s_fog;
extern const m100sim_sword_ref m100sim_s_feeling;
extern const m100sim_sword_ref m100sim_s_pride;
extern const m100sim_sword_ref m100sim_s_monk;
extern const m100sim_sword_ref m100sim_s_abandoned;
extern const m100sim_sword_ref m100sim_s_telecom;
extern const m100sim_sword_ref m100sim_s_glass;
extern const m100sim_sword_ref m100sim_s_crystal;

/* mvoid */
extern const m100sim_sword_ref m100sim_s_heavyV;
extern const m100sim_sword_ref m100sim_s_shieldV;
extern const m100sim_sword_ref m100sim_s_healV;
extern const m100sim_sword_ref m100sim_s_vigorous;

/* private */
m100sim_sword_ref m100sim_s_victory(int n);

# ifdef __cplusplus
} /* extern "C" */
# endif

# endif
