{-# OPTIONS_GHC -fglasgow-exts #-}
{-# LANGUAGE MultiParamTypeClasses #-}
{- |
   License      : BSD3
   Maintainer   : Takano Akio <aljee@hyper.cx>
   Stability    : experimental

   Transformer that adds automatic flash facility to any given port.
   It automatically flushes the buffer after every output operation
   is done.
-}

module System.IO.SSC.AutoFlush
  ( oAutoFlush
  , ioAutoFlush
  , AutoFlush
  ) where

import System.IO.SSC.Classes

newtype AutoFlush d pos a ba ma s = AF s

autoFlush :: s -> IO (AutoFlush d pos a ba ma s)
autoFlush s = return (AF s)

oAutoFlush :: s -> IO (AutoFlush d pos a ba ma s)
oAutoFlush = autoFlush

ioAutoFlush :: s -> IO (AutoFlush d pos a ba ma s)
ioAutoFlush = autoFlush

instance (Port s) => Port (AutoFlush d pos a ba ma s) where
  rawClose (AF s) = rawClose s
  rawIsClosed (AF s) = rawIsClosed s
  rawFlush (AF s) = rawFlush s

instance (Destination d s) => Destination d (AutoFlush d pos a ba ma s) where
  rawDest (AF s) = rawDest s

instance (Lockable s) => Lockable (AutoFlush d pos a ba ma s) where
  sLock (AF s) = sLock s
  sTryLock (AF s) = sTryLock s
  sUnsafeUnlock (AF s) = sUnsafeUnlock s

instance (Discardable s) => Discardable (AutoFlush d pos a ba ma s) where
  rawDiscard (AF s) = rawDiscard s

instance (Seekable pos s) => Seekable pos (AutoFlush d pos a ba ma s) where
  rawGetPos (AF s) = rawGetPos s
  rawSetPos (AF s) = rawSetPos s

instance (RelativeSeekable pos s) => RelativeSeekable pos (AutoFlush d pos a ba ma s) where
  rawRelativeSeek (AF s) = rawRelativeSeek s

instance (SizedInput pos s) => SizedInput pos (AutoFlush d pos a ba ma s) where
  rawInputEnd (AF s) = rawInputEnd s
  rawSeekFromEnd (AF s) = rawSeekFromEnd s

instance (SizedOutput pos s) => SizedOutput pos (AutoFlush d pos a ba ma s) where
  rawOutputEnd (AF s) = rawOutputEnd s

instance (IStream a s) => IStream a (AutoFlush d pos a ba ma s) where
  rawGet (AF s) = rawGet s
  rawGets (AF s) = rawGets s
  rawGetContents (AF s) = rawGetContents s

instance (RetrievableStream a s) => RetrievableStream a (AutoFlush d pos a ba ma s) where
  rawGetUntil (AF s) = rawGetUntil s

instance (OStream a s) => OStream a (AutoFlush d pos a ba ma s) where
  rawPut (AF s) x = rawPut s x >> rawFlush s
  rawPuts (AF s) x = rawPuts s x >> rawFlush s

instance (IBlock ba s) => IBlock ba (AutoFlush d pos a ba ma s) where
  rawGetBufB (AF s) = rawGetBufB s

instance (OBlock ba s) => OBlock ba (AutoFlush d pos a ba ma s) where
  rawPutBufB (AF s) bmode buf size = do
    r <- rawPutBufB s bmode buf size
    rawFlush s
    return r

instance (IMemory ma s) => IMemory ma (AutoFlush d pos a ba ma s) where
  rawReadMem (AF s) = rawReadMem s
  rawEndRead (AF s) = rawEndRead s
  rawFillMemB (AF s) = rawFillMemB s

instance (OMemory ma s) => OMemory ma (AutoFlush d pos a ba ma s) where
  rawWriteMem (AF s) = rawWriteMem s
  -- The memory should not be automatically flushed
  rawEndWrite (AF s) = rawEndWrite s
  rawFlushMemB (AF s) = rawFlushMemB s
