{-# OPTIONS_GHC -fglasgow-exts #-}
{- |
   License      : BSD3
   Maintainer   : Takano Akio <aljee@hyper.cx>
   Stability    : experimental

   Some utility function definitions.
-}
module System.IO.SSC.Utils where

import Control.Concurrent.MVar
import Control.Monad
import Foreign.Storable
import Foreign.Ptr
import System.IO.SSC.Classes
import System.IO.SSC.Internal.Ptr
import System.IO.Error

sPutsLn :: (OStream Char s) => s -> String -> IO ()
sPutsLn s cs = do
  rawPuts s cs
  rawPut s '\n'

sGetLine :: (RetrievableStream Char s) => s -> IO String
sGetLine s = rawGetUntil s '\n'

sUnlock :: (Port s, Lockable s, Discardable s) => s -> IO ()
sUnlock s = do
  rawDiscard s
  rawFlush s
  sUnsafeUnlock s

sElementsInMemory :: (IMemory a s) => s -> IO Int
sElementsInMemory mem = do
  (p0, p1) <- rawReadMem mem
  let r = distancePtr p1 p0
  rawEndRead mem p0
  return r

-- | Test whether the end of input is reached.
sIsEOI :: (IMemory a s) => s -> IO Bool
sIsEOI mem = do
  n <- sElementsInMemory mem
  if n == 0
    then do
      r <- rawFillMemB mem NonBlockingPreferred
      case r of
        Nothing -> return True
        Just _ -> return False
    else return False

sGetBuf :: (IBlock a s) => s -> Ptr a -> Int -> IO Int
sGetBuf b buf size = liftM fst $ rawGetBufB b NonBlockingPreferred buf size

sGetBufNonBlocking :: (IBlock a s) => s -> Ptr a -> Int -> IO (Maybe Int)
sGetBufNonBlocking b buf size = do
  r <- rawGetBufB b NonBlocking buf size
  return $ case r of
    (0, True) -> Just 0
    (0, False) -> Nothing
    (n, _) -> Just n

sPutBuf :: (OBlock a s) => s -> Ptr a -> Int -> IO Int
sPutBuf b buf size = liftM fst $ rawPutBufB b NonBlockingPreferred buf size

sPutBufNonBlocking :: (OBlock a s) => s -> Ptr a -> Int -> IO (Maybe Int)
sPutBufNonBlocking b buf size = do
  r <- rawPutBufB b NonBlocking buf size
  return $ case r of
    (0, True) -> Just 0
    (0, False) -> Nothing
    (n, _) -> Just n

sLookAhead :: (IMemory a m) => m -> IO a
sLookAhead mem = do
  (p0, p1) <- rawReadMem mem
  if p0 == p1
    then do
      rawEndRead mem p0
      r <- rawFillMemB mem NonBlockingPreferred
      case r of
        Nothing -> ioError $ mkIOError eofErrorType "System.IO.SSC.Utils.sLookAhead" Nothing Nothing
        Just n -> do
          (p0', p1') <- rawReadMem mem
          get p0'
    else get p0
  where
    get p0 = do
      e <- peek p0
      rawEndRead mem p0
      return e
