# include "mfile.h"
# include "HsFFI.h"

# include <stdlib.h>
# include <unistd.h>
# include <errno.h>

# ifdef mingw32_HOST_OS

# include <windows.h>

struct mfile_d
{
  void *start;
  HANDLE mapping;
  int voffset;
};

static int get_pagesize(void)
{
  SYSTEM_INFO info;
  GetSystemInfo(&info);
  return info.dwPageSize;
}

void *mfile_map(int fd, off_t off, int size, int write)
{
  void *p;
  HANDLE mapping;
  int pagesize = get_pagesize();
  off_t offset = off / pagesize * pagesize;
  int voff = off - offset;
  struct mfile_d *d = malloc(sizeof(struct mfile_d));
  if(!d)
    return NULL;
  mapping = CreateFileMapping(
    (HANDLE)_get_osfhandle(fd),
    NULL,
    write ? PAGE_READWRITE : PAGE_READONLY,
    0, 0,
    NULL);
  if(!mapping)
  {
    errno = EINVAL; /* FIXME: Use GetLastError */
    free(d);
    return NULL;
  }
  p = MapViewOfFile(
    mapping,
    write ? FILE_MAP_WRITE : FILE_MAP_READ,
# if 0
    (DWORD)(offset >> 32), (DWORD)offset,
# else
    0, (DWORD)offset, /* off_t is 32 bit */
# endif
    size + voff);
  if(!p)
  {
    errno = EINVAL; /* FIXME: Use GetLastError */
    free(d);
    CloseHandle(mapping);
    return NULL;
  }
  d->start = p;
  d->mapping = mapping;
  d->voffset = voff;
  return d;
}

int mfile_unmap(void *p)
{
  int retval = 0;
  struct mfile_d *d = p;
  if(!UnmapViewOfFile(d->start))
  {
    errno = EINVAL; /* FIXME: Use GetLastError */
    retval = -1;
  }
  if(!CloseHandle(d->mapping))
  {
    errno = EINVAL; /* FIXME: Use GetLastError */
    retval = -1;
  }
  return retval;
}

# else /* mingw32_HOST_OS */

# include <sys/mman.h>

struct mfile_d
{
  void *start;
  int length;
  int voffset;
};

void *mfile_map(int fd, off_t off, int size, int write)
{
  int prot;
  void *p;
  int pagesize;
  int offset;
  int voff;
  struct mfile_d *d = malloc(sizeof(struct mfile_d));
  if(!d)
    return NULL;
  if((pagesize = sysconf(_SC_PAGESIZE)) == -1)
    return NULL;
  offset = off / pagesize * pagesize;
  voff = off - offset;
  prot = PROT_READ | (write ? PROT_WRITE : 0);
  p = mmap(NULL, size + voff, prot, MAP_SHARED, fd, offset);
  if(p == MAP_FAILED)
  {
    free(d);
    return NULL;
  }
  d->start = p;
  d->length = size;
  d->voffset = voff;
  return d;
}

int mfile_unmap(void *p)
{
  struct mfile_d *d = p;
  return munmap(d->start, d->length);
}

# endif /* !mingw32_HOST_OS */

void *mfile_ptr(void *p)
{
  struct mfile_d *d = p;
  return (char *)d->start + d->voffset;
}
